// Copyright 2023 Christian Ulrich
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// usage: ./part1 < input

use std::io;

pub struct CalibrationValues {
    inner: Vec<Vec<char>>,
}

impl CalibrationValues {
    pub fn from_stdin() -> Result<Self, &'static str> {
        let mut values = vec![];
        for line in io::stdin().lines() {
            let digits = line
                .unwrap()
                .chars()
                .filter(|c| c.is_digit(10))
                .collect::<Vec<_>>();
            if digits.len() > 0 {
                values.push(digits);
            }
        }
        Ok(Self { inner: values })
    }

    pub fn iter(&mut self) -> std::slice::Iter<'_, Vec<char>> {
        self.inner.iter()
    }
}

fn main() -> Result<(), &'static str> {
    let mut values = CalibrationValues::from_stdin()?;
    let mut sum = 0;
    for digits in values.iter() {
        sum += [*digits.first().unwrap(), *digits.last().unwrap()]
            .iter()
            .collect::<String>()
            .parse::<u32>()
            .map_err(|_| "cannot parse calibration value")?;
    }
    println!("{sum}");
    Ok(())
}
